﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using WrapperPatternExample.Utils;

namespace WrapperPatternExample.Wrappers
{
	public enum HideSide
	{
		Top,
		Bottom,
		Left,
		Right
	}
	public class ElementHidderWrapper : FrameworkElement
	{
		public double MinMargin
		{
			get
			{
				return (double)GetValue(MinMarginProperty);
			}
			set
			{
				SetValue(MinMarginProperty, value);
			}
		}

		// Using a DependencyProperty as the backing store for MaxShow.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty MinMarginProperty =
			DependencyProperty.Register("MinMargin", typeof(double), typeof(ElementHidderWrapper), Helper.CreateMetadata(0.0, MinMarginChanged));


		private static void MinMarginChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
		{
			ElementHidderWrapper hidder = (ElementHidderWrapper)sender;
			hidder.UpdateElement();
		}



		public HideSide HideSide
		{
			get
			{
				return (HideSide)GetValue(HideSideProperty);
			}
			set
			{
				SetValue(HideSideProperty, value);
			}
		}

		// Using a DependencyProperty as the backing store for HideSide.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty HideSideProperty =
			DependencyProperty.Register("HideSide", typeof(HideSide), typeof(ElementHidderWrapper), Helper.CreateMetadata(HideSide.Bottom));




		public double Show
		{
			get
			{
				return (double)GetValue(ShowProperty);
			}
			set
			{
				SetValue(ShowProperty, value);
			}
		}

		// Using a DependencyProperty as the backing store for Show.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty ShowProperty =
			DependencyProperty.Register("Show", typeof(double), typeof(ElementHidderWrapper), Helper.CreateMetadata(1.0, ShowChanged));




		public FrameworkElement Element
		{
			get
			{
				return (FrameworkElement)GetValue(ElementProperty);
			}
			set
			{
				SetValue(ElementProperty, value);
			}
		}

		// Using a DependencyProperty as the backing store for Element.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty ElementProperty =
			DependencyProperty.Register("Element", typeof(FrameworkElement), typeof(ElementHidderWrapper), Helper.CreateMetadata(null, ElementChanged));

		private static void ElementChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
		{
			ElementHidderWrapper hidder = (ElementHidderWrapper)sender;
			FrameworkElement oldValue = args.OldValue as FrameworkElement;
			FrameworkElement newValue = args.NewValue as FrameworkElement;
			if(oldValue != null)
				oldValue.SizeChanged -= hidder.SizeChanged;
			if(newValue != null)
				newValue.SizeChanged += hidder.SizeChanged;
			hidder.UpdateElement();
		}

		private void SizeChanged(object sender, SizeChangedEventArgs e)
		{
			UpdateElement();
		}

		private static void ShowChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
		{
			ElementHidderWrapper hidder = (ElementHidderWrapper)sender;
			hidder.UpdateElement();
		}

		private void UpdateElement()
		{
			if(Element == null)
				return;
			var maxValue = GetMaxShowValue(Element);
			var minValue = MinMargin;
			var calculatedShowValue = minValue + (maxValue - minValue) * Show;
			var marginValue = maxValue - calculatedShowValue;
			SetMarginValue(Element, marginValue);

		}

		private void SetMarginValue(FrameworkElement element, double marginValue)
		{
			if(HideSide == HideSide.Left)
			{
				element.Margin = new Thickness(-marginValue, element.Margin.Top, element.Margin.Right, element.Margin.Bottom);
			}
			else if(HideSide == HideSide.Top)
			{
				element.Margin = new Thickness(element.Margin.Left, -marginValue, element.Margin.Right, element.Margin.Bottom);
			}
			else if(HideSide == HideSide.Right)
			{
				element.Margin = new Thickness(element.Margin.Left, element.Margin.Top, -marginValue, element.Margin.Bottom);
			}
			else if(HideSide == HideSide.Bottom)
			{
				element.Margin = new Thickness(element.Margin.Left, element.Margin.Top, element.Margin.Right, -marginValue);
			}
		}

		private double GetMaxShowValue(FrameworkElement element)
		{
			if(HideSide == HideSide.Bottom || HideSide == HideSide.Top)
				return element.ActualHeight;
			else
				return element.ActualWidth;
		}


	}
}
