﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using WrapperPatternExample.Utils;

namespace WrapperPatternExample.Wrappers
{
	public class SizeWrapper : FrameworkElement
	{
		public FrameworkElement Element
		{
			get
			{
				return (FrameworkElement)GetValue(ElementProperty);
			}
			set
			{
				SetValue(ElementProperty, value);
			}
		}

		public double RealHeight
		{
			get
			{
				return (double)GetValue(RealHeightProperty);
			}
			set
			{
				SetValue(RealHeightProperty, value);
			}
		}

		// Using a DependencyProperty as the backing store for RealHeight.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty RealHeightProperty =
			DependencyProperty.Register("RealHeight", typeof(double), typeof(SizeWrapper), Helper.CreateMetadata(0.0));




		public double RealWidth
		{
			get
			{
				return (double)GetValue(RealWidthProperty);
			}
			set
			{
				SetValue(RealWidthProperty, value);
			}
		}

		// Using a DependencyProperty as the backing store for RealWidth.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty RealWidthProperty =
			DependencyProperty.Register("RealWidth", typeof(double), typeof(SizeWrapper), Helper.CreateMetadata(0.0));


		// Using a DependencyProperty as the backing store for Element.  This enables animation, styling, binding, etc...
		public static readonly DependencyProperty ElementProperty =
			DependencyProperty.Register("Element", typeof(FrameworkElement), typeof(SizeWrapper), Helper.CreateMetadata(null, OnElementChanged));

		private static void OnElementChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
		{
			var wrapper = (SizeWrapper)sender;
			var oldElement = args.OldValue as FrameworkElement;
			var newElement = args.NewValue as FrameworkElement;
			if(oldElement != null)
				oldElement.SizeChanged -= wrapper.SizeChanged;
			if(newElement != null)
			{
				newElement.SizeChanged += wrapper.SizeChanged;
				wrapper.UpdateSize(new Size(newElement.ActualWidth, newElement.ActualHeight));
			}
		}


		void SizeChanged(object sender, SizeChangedEventArgs e)
		{
			UpdateSize(e.NewSize);
		}

		private void UpdateSize(Size size)
		{
			RealHeight = size.Height;
			RealWidth = size.Width;
		}
	}
}
